package javax.naming;

import java.util.Enumeration;
import java.util.Vector;
import java.io.Serializable;

public class Reference implements Cloneable, Serializable
{
    protected String className;
    protected Vector<RefAddr> addrs;
    protected String classFactory;
    protected String classFactoryLocation;
    private static final long serialVersionUID = -1673475790065791735L;
    
    public Reference(String className) {
        this.addrs = null;
        this.classFactory = null;
        this.classFactoryLocation = null;
        this.className = className;
        this.addrs = new Vector();
    }
    
    public Reference(String className, RefAddr obj) {
        this.addrs = null;
        this.classFactory = null;
        this.classFactoryLocation = null;
        this.className = className;
        (this.addrs = new Vector()).addElement(obj);
    }
    
    public Reference(String s, String classFactory, String classFactoryLocation) {
        this(s);
        this.classFactory = classFactory;
        this.classFactoryLocation = classFactoryLocation;
    }
    
    public Reference(String s, RefAddr refAddr, String classFactory, String classFactoryLocation) {
        this(s, refAddr);
        this.classFactory = classFactory;
        this.classFactoryLocation = classFactoryLocation;
    }
    
    public String getClassName() {
        return this.className;
    }
    
    public String getFactoryClassName() {
        return this.classFactory;
    }
    
    public String getFactoryClassLocation() {
        return this.classFactoryLocation;
    }
    
    public RefAddr get(String anotherString) {
        int size = this.addrs.size();
        int i = 0;
        while (i < size) {
            RefAddr refAddr = (RefAddr)this.addrs.elementAt(i);
            if (refAddr.getType().compareTo(anotherString) == 0) {
                return refAddr;
            }
            ++i;
        }
        return null;
    }
    
    public RefAddr get(int index) {
        return (RefAddr)this.addrs.elementAt(index);
    }
    
    public Enumeration<RefAddr> getAll() {
        return this.addrs.elements();
    }
    
    public int size() {
        return this.addrs.size();
    }
    
    public void add(RefAddr obj) {
        this.addrs.addElement(obj);
    }
    
    public void add(int index, RefAddr obj) {
        this.addrs.insertElementAt(obj, index);
    }
    
    public Object remove(int n) {
        Object element = this.addrs.elementAt(n);
        this.addrs.removeElementAt(n);
        return element;
    }
    
    public void clear() {
        this.addrs.setSize(0);
    }
    
    @Override
    public boolean equals(Object o) {
        if (o != null && o instanceof Reference) {
            Reference reference = (Reference)o;
            if (reference.className.equals(this.className) && reference.size() == this.size()) {
                Enumeration<RefAddr> all = this.getAll();
                Enumeration<RefAddr> all2 = reference.getAll();
                while (all.hasMoreElements()) {
                    if (((RefAddr)all.nextElement()).equals(all2.nextElement())) {
                        continue;
                    }
                    return false;
                }
                return true;
            }
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        int hashCode = this.className.hashCode();
        Enumeration<RefAddr> all = this.getAll();
        while (all.hasMoreElements()) {
            hashCode += ((RefAddr)all.nextElement()).hashCode();
        }
        return hashCode;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("Reference Class Name: " + this.className + "\n");
        for (int size = this.addrs.size(), i = 0; i < size; ++i) {
            sb.append(this.get(i).toString());
        }
        return sb.toString();
    }
    
    public Object clone() {
        Reference reference = new Reference(this.className, this.classFactory, this.classFactoryLocation);
        Enumeration<RefAddr> all = this.getAll();
        reference.addrs = new Vector();
        while (all.hasMoreElements()) {
            reference.addrs.addElement(all.nextElement());
        }
        return reference;
    }
}
